function zi = PolylineIntersectSegment(seg,pline)
%  function zi = PolylineIntersectSegment(seg,pline)
%  Intersections of two dimension line segment with a polyline. 
%  A polyline is a set of connected line segments. The segments
%  are not necessarily closed i.e the end point is not equal the start
%  although they can be. The points of the segment and the polyline
%  are represented as complex numbers with coordinates x + iy. 
%  see the discussion at http://aprendtech.com/wordpress/?p=140
%
%  The code fully accomodates vertical lines; it does NOT use y = ax + b
%  to represent the lines. 
%  inputs:
%     seg: a length two complex array of the endpoints of the line segment
%     pline: a 1D complex array of the start and end points of the polyline. The segments
%        are connected so the end point of one is the start point of the next
%  outputs:
%     zi: the intersection points of the segment with the polyline. Empty if no intersect. 
%     
%  example:   
%  zseg = [1, 1+1i];
%  psquare = [0 1, 1+1i, 1i, 0];
%  poff = 1.5*(pline - (0.5+0.5*1i)); % offset square
%  zi = PolylineIntersectSegment(zseg,poff);
%  plot(poff,'-b')
%  hold on,plot(zseg,'-r'),hold off
%  hold on,plot(zi,'ok'),hold off
%  
%  REA: 2004-Jul-8 -> 2011-Aug-30 10:41  


assert(numel(seg)==2);
assert(numel(pline)>=2);
   % will use column vectors
seg = seg(:); 
pline = pline(:); 
nlines = numel(pline) - 1;

    % see discussion of formulas in my blog post: http://aprendtech.com/wordpress/?p=140
    % convert to notation corresponding to my equations to describe the line segments: 
    % single line segment is P1 + s*d1 and each polyline seg is P2 + t*d2
    % for intersect points in the segments 0<=s<=1 and  0<=t<=1 
P1 = seg(1)*ones(nlines,1);
d1  = (seg(2) - seg(1) )*ones(nlines,1);   
P2 = pline(1:end-1);
d2 = diff(pline);

  % rotate the diff vectors to form perpendicular vectors
ns_1 = 1i*d1;
ns_2 = 1i*d2;

  % the difference vectors of the initial points
Ds = P2 - P1;

   % the offsets to the intersections
s = zdot(Ds,ns_2)./zdot(d1,ns_2);
t = -zdot(Ds,ns_1)./zdot(d2,ns_1);

intersectsOK = (s>=0)&(s<1)&(t>=0)&(t<1);

    % find points of intersect, if any
if any(intersectsOK)
   zi = seg(1) + s(intersectsOK)*d1(1); % all d1s are the same so use the first one
else    
   zi = [];
end


     % ---- LOCAL FUNCTIONS ------
       
     function d = zdot(z1,z2) % dot product for complex vectors
    d = real( z1(:) .* conj(z2(:)));